import 'dart:convert';
import 'dart:io';

import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:flutter/material.dart';
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:shared_preferences/shared_preferences.dart';

class AuthProvider with ChangeNotifier {
  Future<Map<String, dynamic>> login(String loginResponseBody) async {
    Map<String, dynamic> rsBody = json.decode(loginResponseBody);
    final token = rsBody["jwtToken"];
    if (token == null) {
      return {};
    }

    Map<String, dynamic> tokenData = JwtDecoder.decode(token);
    await setToken(token);
    UserType role = UserType.from[tokenData["Role"]];
    return {
      'success': true,
      'status': HttpStatus.ok,
      'accessToken': token,
      'role': role
    };
  }

  Future<void> logout() async {
    final preferences = await SharedPreferences.getInstance();
    preferences.remove(TOKEN_KEY);
    notifyListeners();
  }

  Future<String?> getToken() async {
    final preferences = await SharedPreferences.getInstance();
    String? token = preferences.getString(TOKEN_KEY);
    return token;
  }

  Future<Map<String, dynamic>> getTokenData() async {
    final token = await getToken();
    if (token == null) return {};
    final data = JwtDecoder.decode(token);
    if (data["Role"] != null) {
      data["role"] = UserType.from[data["Role"]];
    }

    return data;
  }

  Future<bool> isAuthenticated() async {
    final token = await getTokenData();
    if (token.isEmpty) {
      return false;
    }
    final expTimestamp = token["exp"] ?? 0;
    final expireDate = DateTime.fromMillisecondsSinceEpoch(expTimestamp * 1000);
    final isExpired = expireDate.isBefore(DateTime.now());
    return !isExpired;
  }

  Future<void> setToken(String token) async {
    final preferences = await SharedPreferences.getInstance();
    preferences.setString(TOKEN_KEY, token);
    notifyListeners();
  }

  Future<Map<String, String>> getHttpHeaders() async {
    final headers = {"Content-Type": "application/json"};

    final token = await this.getToken();
    if (token != null) {
      headers.addAll({HttpHeaders.authorizationHeader: "Bearer $token"});
    }

    return headers;
  }
}
