import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/OrderItemModel.dart';
import 'package:apus_drones_mobile/models/OrderModel.dart';
import 'package:apus_drones_mobile/providers/ActiveBarItemProvider.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/OrdersScreen.dart';
import 'package:apus_drones_mobile/screens/PaymentScreen.dart';
import 'package:apus_drones_mobile/services/MaxWeightService.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:apus_drones_mobile/widgets/BotNavBar.dart';
import 'package:apus_drones_mobile/widgets/CartItem.dart';
import 'package:apus_drones_mobile/widgets/Fee.dart';
import 'package:apus_drones_mobile/widgets/WeightAlert.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class CartScreen extends StatefulWidget {
  static final String routeName = 'Cart';
  static final int botNavItemIndex = 1;

  @override
  _CartPageState createState() => _CartPageState();
}

class _CartPageState extends State<CartScreen> {
  OrderModel? cart;
  OrderService service = new OrderService();
  MaxWeightService maxWeightService = new MaxWeightService();
  double maxWeight = 0;

  _loadCart() {
    service.myCart().then((order) => this.setState(() {
          cart = order;
        }));
    maxWeightService.getMaxWeight().then((max) => this.setState(() {
          maxWeight = max;
        }));
  }

  _checkout() async {
    final _cart = cart;
    if (_cart == null) {
      return;
    }

    final newCart = OrderModel(
      id: _cart.id,
      partner: _cart.partner,
      customer: _cart.customer,
      status: _cart.status,
      createdAt: _cart.createdAt,
      items: _cart.items,
      deliveryPrice: _cart.deliveryPrice,
      orderPrice: _cart.orderPrice,
    );

    try {
      await service.updateCart(newCart);
      Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) => PaymentScreen(),
        ),
      );
    } catch (error) {}
  }

  @override
  void initState() {
    super.initState();
    Future.delayed(Duration.zero, () async {
      this._loadCart();
      Provider.of<ActiveBarItemProvider>(context, listen: false)
          .setActiveIndex(2);
    });
  }

  @override
  Widget build(BuildContext context) {
    final _cart = cart;
    return Scaffold(
      appBar: Appbar(title: 'Carrinho'),
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.grey.shade100,
      body: SafeArea(
        child: _cart != null && _cart.items.length > 0
            ? Column(
                children: [
                  Expanded(
                    child: ListView(
                      children: <Widget>[createCartList(context)],
                    ),
                  ),
                  footer(context)
                ],
              )
            : Center(
                child: Text(
                  'Seu carrinho está vazio',
                  style: TextStyle(fontSize: 24),
                ),
              ),
      ),
      bottomNavigationBar: BotNavBar(
        role: UserType.CUSTOMER,
      ),
    );
  }

  footer(BuildContext context) {
    return Container(
      padding: EdgeInsets.only(bottom: 30),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.end,
        children: <Widget>[
          Fee(
              productsSumPrice: this._sumProductsPrices(),
              deliveryFee: cart != null ? cart!.deliveryPrice : 0,
              totalPrice: cart != null
                  ? cart!.deliveryPrice + this._sumProductsPrices()
                  : 0),
          SizedBox(height: 50),
          Align(
            alignment: Alignment.bottomCenter,
            child: ElevatedButton(
              onPressed: () {
                _checkout();
              },
              style: ElevatedButton.styleFrom(
                  padding:
                      EdgeInsets.only(top: 12, left: 60, right: 60, bottom: 12),
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.all(Radius.circular(4)))),
              child: Text(
                "Prosseguir para pagamento",
                style: TextStyle(
                  fontSize: 16,
                  color: Colors.white,
                ),
              ),
            ),
          ),
        ],
      ),
      margin: EdgeInsets.only(top: 16),
    );
  }

  createCartList(BuildContext context) {
    if (cart == null) {
      return Text('');
    }

    return ListView.builder(
      shrinkWrap: true,
      primary: false,
      itemBuilder: (context, index) => CartItem(
          key: Key(cart!.items[index].id.toString()),
          item: cart!.items[index],
          storeName: cart!.partner.name,
          removeCallback: this.updateItem,
          isWeightValidCallback: this.isWeightValid,
          updateItemCallback: this.updateItem),
      itemCount: cart!.items.length,
    );
  }

  removeItem(OrderItemModel item) {
    var itemIndex =
        this.cart!.items.indexWhere((inCart) => inCart.id == item.id);
    this.setState(() {
      cart!.items.removeAt(itemIndex);
    });
  }

  bool isWeightValid(OrderItemModel item) {
    var otherItems =
        this.cart!.items.where((inCart) => inCart.id != item.id).toList();
    var updatedItemWeight = item.quantity * item.product.weight;
    var isValid = updatedItemWeight < maxWeight;
    if (otherItems.length != 0) {
      var otherItemsWeigth = otherItems
          .map((inCart) => inCart.quantity * inCart.product.weight)
          .reduce((acc, next) => acc + next);
      isValid = otherItemsWeigth + updatedItemWeight < maxWeight;
    }
    if (!isValid) {
      WeightAlert.show(context);
    }
    return isValid;
  }

  void updateItem(OrderItemModel item) async {
    var itemIndex =
        this.cart!.items.indexWhere((inCart) => inCart.id == item.id);
    this.cart!.items.replaceRange(itemIndex, itemIndex + 1, [item]);
    try {
      await this.service.updateCart(this.cart!);
      this.service.myCart().then((value) => this.setState(() {
            cart = value;
          }));
    } catch (e) {
      WeightAlert.show(context);
      this.setState(() {
        cart = null;
      });
      this.service.myCart().then((value) => this.setState(() {
            cart = value;
          }));
    }
  }

  double _sumProductsPrices() {
    if (cart == null) return 0;
    return this
        .cart!
        .items
        .fold(0, (acc, item) => item.price * item.quantity + acc);
  }

  checkOutAlertDialog(BuildContext context) {
    Widget accompanyOrderButton = ElevatedButton(
      style: ElevatedButton.styleFrom(primary: Colors.red),
      child: Text("Acompanhar Pedido"),
      onPressed: () {
        //Ir para a tela de acompanhar pedido
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => OrdersScreen(),
          ),
        );
      },
    );

    AlertDialog alertPopUp = AlertDialog(
      title: Text("Compra Finalizada!"),
      content: Text(
          "Obrigado por escolher a Apus Drones. Logo logo seu pedido chegará até você, mas até lá, o próximo passo é você acompanhar o status do seu pedido."),
      actions: [
        accompanyOrderButton,
      ],
    );
    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alertPopUp;
      },
    );
  }
}
