import 'dart:math';

import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/AddressModel.dart';
import 'package:apus_drones_mobile/models/CreateCustomer.dart';
import 'package:apus_drones_mobile/models/CreatePartner.dart';
import 'package:apus_drones_mobile/screens/LoginScreen.dart';
import 'package:apus_drones_mobile/services/AuthService.dart';
import 'package:apus_drones_mobile/services/LocationService.dart';
import 'package:apus_drones_mobile/widgets/AddressAutocomplete.dart';
import 'package:flutter/material.dart';

import 'package:places_service/places_service.dart';

class CreateUserScreen extends StatefulWidget {
  final UserType? role;
  const CreateUserScreen({Key? key, this.role}) : super(key: key);

  @override
  _CreateUserScreenState createState() => _CreateUserScreenState();
}

class _CreateUserScreenState extends State<CreateUserScreen> {
  late CreateCustomer customer;
  late CreatePartner partner;
  TextEditingController nameController = TextEditingController();
  TextEditingController cpfController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  TextEditingController emailController = TextEditingController();
  TextEditingController cepController = TextEditingController();
  TextEditingController addressController = TextEditingController();
  LocationService locationService = LocationService();
  List<PlacesAutoCompleteResult> resultPlaces = [];
  AddressModel? selectedAddress;
  AuthService authService = new AuthService();
  String? addressErrorText;
  ScrollController scrollController = ScrollController();

  saveCustomer() async {
    await authService.signupCustomer(customer);
  }

  savePartner() async {
    await authService.signupPartner(partner);
  }

  _onAddressSelected(AddressModel address) {
    this.selectedAddress = address;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: BodyCreateUser(),
    );
  }

  Widget BodyCreateUser() {
    return Container(
      width: MediaQuery.of(context).size.width,
      height: MediaQuery.of(context).size.height,
      child: Padding(
        padding: const EdgeInsets.only(top: 70, right: 25, left: 25),
        child: Column(
          children: [
            Text("Cadastro do Cliente",
                style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold)),
            Expanded(
              child: SingleChildScrollView(
                controller: scrollController,
                child: getColumn(),
              ),
            ),
            SizedBox(
              height: 30,
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.end,
              children: [
                ElevatedButton(
                  onPressed: () {
                    back();
                  },
                  child: Text('Cancelar'),
                ),
                SizedBox(
                  width: 20,
                ),
                ElevatedButton(
                    onPressed: () {
                      createUser();
                    },
                    child: Text('Criar')),
              ],
            ),
          ],
        ),
      ),
    );
  }

  Column getColumn() {
    List<Widget> list = [];

    list.add(SizedBox(
      height: 30,
    ));
    list.add(TextField(
      controller: nameController,
      decoration: InputDecoration(
        labelText: 'Nome completo',
        border: OutlineInputBorder(),
        helperText: 'Este nome aparecerá para os parceiros',
      ),
    ));
    list.add(SizedBox(
      height: 30,
    ));
    list.add(TextField(
      controller: cpfController,
      decoration: InputDecoration(
        labelText: 'CPF',
        border: OutlineInputBorder(),
      ),
      keyboardType: TextInputType.number,
    ));
    list.add(SizedBox(
      height: 30,
    ));
    list.add(TextField(
      controller: passwordController,
      decoration: InputDecoration(
        labelText: 'Senha',
        border: OutlineInputBorder(),
      ),
      keyboardType: TextInputType.visiblePassword,
    ));
    list.add(SizedBox(
      height: 30,
    ));
    //Usar o FileUploadInputElement() talvez para fazer upload de imagem
    list.add(TextField(
      controller: emailController,
      decoration: InputDecoration(
        labelText: 'Email',
        border: OutlineInputBorder(),
      ),
      keyboardType: TextInputType.emailAddress,
    ));
    list.add(SizedBox(
      height: 30,
    ));
    list.add(
      AddressAutocomplete(
        placeholder: "Endereço",
        onAddressSelected: _onAddressSelected,
        errorText: this.addressErrorText,
        scrollController: scrollController,
      ),
    );
    return new Column(children: list);
  }

  void createUser() {
    if (this.selectedAddress?.number == -1) {
      setState(() {
        this.addressErrorText = "Informe o número do endereço.";
      });
      return;
    } else {
      this.addressErrorText = null;
    }

    switch (this.widget.role) {
      case UserType.CUSTOMER:
        this.createCliente();
        break;
      case UserType.PARTNER:
        this.createPartner();
        break;
      case UserType.PILOT:
        break;
      default:
        throw new ArgumentError.notNull("Papel do usuário não pode ser nulo.");
    }
  }

  Future<void> createCliente() async {
    var rng = new Random();
    int n = rng.nextInt(300) + 1;
    String url =
        "https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/$n.png";
    customer = CreateCustomer(
      selectedAddress!,
      url,
      cpfController.text,
      emailController.text,
      nameController.text,
      passwordController.text,
    );

    await saveCustomer();
    back();
  }

  void createPartner() {
    var rng = new Random();
    int n = rng.nextInt(300) + 1;
    String url =
        "https://raw.githubusercontent.com/PokeAPI/sprites/master/sprites/pokemon/$n.png";
    print('url: ' + url);
    partner = CreatePartner(
      url,
      cpfController.text,
      emailController.text,
      nameController.text,
      passwordController.text,
      selectedAddress!,
    );

    savePartner();
    print("Parceiro cadastrado!");
    back();
  }

  void back() {
    Navigator.push(
        context, MaterialPageRoute(builder: (context) => LoginScreen()));
  }
}
