import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/PartnerModel.dart';
import 'package:apus_drones_mobile/models/UserModel.dart';
import 'package:apus_drones_mobile/screens/Appbar.dart';
import 'package:apus_drones_mobile/screens/UserEditProfileScreen.dart';
import 'package:apus_drones_mobile/services/PartnerService.dart';
import 'package:apus_drones_mobile/widgets/UserDisplayedProperty.dart';
import 'package:flutter/material.dart';

class PartnerProfile extends StatefulWidget {
  const PartnerProfile({Key? key}) : super(key: key);

  @override
  _PartnerProfileState createState() => _PartnerProfileState();
}

class _PartnerProfileState extends State<PartnerProfile> {
  PartnerModel? partner;
  PartnerService partnerService = PartnerService();

  _PartnerProfileState() {
    _loadProfile();
  }

  _loadProfile() {
    partnerService.getProfile().then((partner) => onPartnerData(partner));
  }

  onPartnerData(PartnerModel data) {
    this.setState(() {
      this.partner = data;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: Appbar(
        title: "Parceiro - ${this.partner?.name ?? ''}",
      ),
      body: Column(
        children: [
          Padding(
            padding: EdgeInsets.all(20),
            child: Row(mainAxisAlignment: MainAxisAlignment.center, children: [
              Container(
                width: 140.0,
                height: 140.0,
                decoration: BoxDecoration(
                    shape: BoxShape.circle,
                    border: Border.all(width: 3.0, color: Colors.grey),
                    image: DecorationImage(
                        fit: BoxFit.fill,
                        image:
                            NetworkImage(partner?.avatarUrl ?? IMG_NO_PHOTO))),
              ),
            ]),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text(
                "${partner?.name}",
                style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold),
              )
            ],
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              Text(
                "${partner?.email}",
                style: TextStyle(fontSize: 16),
              )
            ],
          ),
          Padding(
            padding: EdgeInsets.all(16),
            child: Padding(
              padding: EdgeInsets.only(top: 24),
              child: Column(
                children: [
                  Padding(
                    padding: EdgeInsets.all(8),
                    child: UserDisplayedProperty<String?>(
                      name: "CPF / CNPJ",
                      value: partner?.cpfCnpj,
                    ),
                  ),
                  Padding(
                    padding: EdgeInsets.all(8),
                    child: UserDisplayedProperty<String?>(
                      name: "Endereço",
                      value: (partner?.address?.toString()) ??
                          "Endereço não cadastrado",
                    ),
                  )
                ],
              ),
            ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton.extended(
        onPressed: () {
          final user = new UserModel(
              id: partner!.id,
              name: partner!.name,
              email: partner!.email ?? "",
              cpfCnpj: partner!.cpfCnpj ?? "",
              avatarUrl: partner!.avatarUrl,
              role: UserType.PARTNER,
              address: partner!.address);
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => UserEditProfileScreen(customer: user),
            ),
          );
        },
        label: Icon(Icons.edit),
        shape: CircleBorder(),
      ),
    );
  }
}
