import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/constants/UserType.dart';
import 'package:apus_drones_mobile/models/AddressModel.dart';
import 'package:apus_drones_mobile/models/UserModel.dart';
import 'package:apus_drones_mobile/screens/CustomerProfile.dart';
import 'package:apus_drones_mobile/screens/PartnerProfile.dart';
import 'package:apus_drones_mobile/services/CustomerService.dart';
import 'package:apus_drones_mobile/services/ProfileService.dart';
import 'package:apus_drones_mobile/widgets/AddressAutocomplete.dart';
import 'package:flutter/material.dart';

class UserEditProfileScreen extends StatefulWidget {
  final UserModel? customer;
  UserEditProfileScreen({Key? key, required this.customer}) : super(key: key);

  @override
  _UserEditProfileScreenState createState() => _UserEditProfileScreenState();
}

class _UserEditProfileScreenState extends State<UserEditProfileScreen> {
  TextEditingController nameController = TextEditingController();
  TextEditingController cpfController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  TextEditingController emailController = TextEditingController();
  AddressModel? selectedAddress;
  String? addressErrorText;

  late bool passwordVisibility;

  UserModel? _customer;

  CustomerService customerService = CustomerService();
  ProfileService profileService = ProfileService();

  @override
  void initState() {
    super.initState();

    final _currentCustomer = widget.customer;
    this.setState(() {
      _customer = _currentCustomer;
      if (_currentCustomer != null) {
        nameController.value = TextEditingValue(text: _currentCustomer.name);
        cpfController.value = TextEditingValue(text: _currentCustomer.cpfCnpj);
        emailController.value = TextEditingValue(text: _currentCustomer.email);
      }
      passwordVisibility = false;
    });
  }

  void updateCustomer() async {
    final _currentCustomer = _customer;
    if (_currentCustomer == null) {
      return;
    }

    UserModel newCustomer = UserModel(
      id: _currentCustomer.id,
      name: nameController.text,
      email: emailController.text,
      cpfCnpj: cpfController.text,
      avatarUrl: _currentCustomer.avatarUrl,
      role: _currentCustomer.role,
      address: selectedAddress ?? _currentCustomer.address,
    );

    try {
      await profileService.updateProfile(newCustomer);
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => newCustomer.role == UserType.PARTNER
                  ? PartnerProfile()
                  : CustomerProfile()));
    } catch (error) {
      print(error);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        leading: IconButton(
            onPressed: () async {
              Navigator.of(context).pushNamed('LoginScreen');
            },
            icon: Icon(
              Icons.arrow_back_rounded,
              color: Colors.black,
              size: 24,
            )),
        title: Text('Editar Perfil'),
      ),
      body: SingleChildScrollView(
        child: GestureDetector(
          onTap: () {
            FocusScope.of(context).unfocus();
          },
          child: Padding(
            padding: EdgeInsets.only(top: 15),
            child: Column(
              children: [
                fotoPerfil(),
                Column(
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    inputUsuarioTxt(
                        "Nome", "Ex: João da Silva", nameController),
                    inputUsuarioNumber(
                        "CPF", "Ex: 123.456.789-00", cpfController),
                    inputUsuarioTxt(
                        "Email", "Ex: exemplo@gmail.com", emailController),
                    inputAddress(),
                    inputUsuarioSenha("Senha", passwordController),
                    botoesRodape(),
                  ],
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  fotoPerfil() {
    return Center(
      child: Stack(
        children: [
          Container(
            width: 130,
            height: 130,
            decoration: BoxDecoration(
              border: Border.all(
                  width: 4, color: Theme.of(context).scaffoldBackgroundColor),
              boxShadow: [
                BoxShadow(
                  spreadRadius: 2,
                  blurRadius: 8,
                  color: Colors.black.withOpacity(0.1),
                )
              ],
              shape: BoxShape.circle,
              image: DecorationImage(
                image: NetworkImage(_customer?.avatarUrl ?? IMG_NO_PHOTO),
              ),
            ),
          ),
          Positioned(
            bottom: 0,
            right: 0,
            child: Container(
              height: 40,
              width: 40,
              decoration: BoxDecoration(
                  color: Colors.red,
                  shape: BoxShape.circle,
                  border: Border.all(
                      width: 4,
                      color: Theme.of(context).scaffoldBackgroundColor)),
              child: IconButton(
                padding: EdgeInsets.all(0),
                onPressed: () {
                  print('Button Pressed');
                },
                icon: Icon(
                  Icons.edit,
                  color: Colors.white,
                ),
              ),
            ),
          )
        ],
      ),
    );
  }

  inputUsuarioTxt(
      String labelText, String hintText, TextEditingController controller) {
    return Padding(
      padding: EdgeInsets.only(left: 20, top: 25, right: 20, bottom: 0),
      child: TextFormField(
        controller: controller,
        obscureText: false,
        decoration: getInputDecoration(labelText, false, null, hintText),
        style: TextStyle(
          fontFamily: 'Poppins',
        ),
        keyboardType: TextInputType.name,
      ),
    );
  }

  inputUsuarioNumber(
      String labelText, String hintText, TextEditingController controller) {
    return Padding(
      padding: EdgeInsets.only(left: 20, top: 25, right: 20, bottom: 0),
      child: TextFormField(
        controller: controller,
        obscureText: false,
        decoration: getInputDecoration(labelText, false, null, hintText),
        style: TextStyle(
          fontFamily: 'Poppins',
        ),
        keyboardType: TextInputType.number,
      ),
    );
  }

  inputAddress() {
    return Padding(
      padding: EdgeInsetsDirectional.fromSTEB(20, 15, 20, 0),
      child: AddressAutocomplete(
        placeholder: "Endereço",
        decoration:
            this.getInputDecoration("Endereço", false, addressErrorText, null),
        initialValue: this._customer?.address?.toString() ?? '',
        onAddressSelected: _onAddressSelected,
        errorText: addressErrorText,
      ),
    );
  }

  _onAddressSelected(AddressModel address) {
    if (address.number == -1) {
      setState(() {
        addressErrorText = "Informe o número do endereço.";
      });
      return;
    }
    setState(() {
      addressErrorText = "";
    });
    this.selectedAddress = address;
  }

  inputUsuarioSenha(String labelText, TextEditingController controller) {
    return Padding(
      padding: EdgeInsetsDirectional.fromSTEB(20, 15, 20, 0),
      child: TextFormField(
        controller: controller,
        obscureText: !passwordVisibility,
        decoration: this.getInputDecoration("Senha", true, null, null),
        style: TextStyle(
          fontFamily: 'Poppins',
        ),
        keyboardType: TextInputType.visiblePassword,
      ),
    );
  }

  botoesRodape() {
    return Padding(
      padding: EdgeInsets.only(top: 50),
      child: Row(mainAxisAlignment: MainAxisAlignment.spaceAround, children: [
        ElevatedButton(
          onPressed: () {
            print('Button pressed ...');
            Navigator.pop(context);
          },
          child: Text(
            'CANCELAR',
            style: TextStyle(
              fontFamily: 'Poppins',
              color: Color(0xFFFB0808),
              fontSize: 15,
              fontWeight: FontWeight.w500,
            ),
          ),
          style: ElevatedButton.styleFrom(
            primary: Colors.white,
            fixedSize: Size(150, 40),
            elevation: 3,
            shape:
                RoundedRectangleBorder(borderRadius: BorderRadius.circular(20)),
          ),
        ),
        ElevatedButton(
          onPressed: () {
            print('Button pressed ...');
            updateCustomer();
          },
          child: Text(
            'SALVAR',
            style: TextStyle(
              fontFamily: 'Poppins',
              color: Colors.white,
              fontSize: 15,
              fontWeight: FontWeight.w500,
            ),
          ),
          style: ElevatedButton.styleFrom(
            primary: Color(0xFFFB0808),
            fixedSize: Size(150, 40),
            elevation: 3,
            shape:
                RoundedRectangleBorder(borderRadius: BorderRadius.circular(20)),
          ),
        ),
      ]),
    );
  }

  getInputDecoration(
      String labelText, bool showIcon, String? errorText, String? hintText) {
    return InputDecoration(
      labelText: labelText,
      labelStyle: TextStyle(
        fontFamily: 'Poppins',
      ),
      enabledBorder: UnderlineInputBorder(
        borderSide: BorderSide(
          color: Color(0x00000000),
          width: 1,
        ),
        borderRadius: BorderRadius.circular(10),
      ),
      focusedBorder: UnderlineInputBorder(
        borderSide: BorderSide(
          color: Color(0x00000000),
          width: 1,
        ),
        borderRadius: BorderRadius.circular(10),
      ),
      hintText: hintText,
      filled: true,
      errorText: errorText,
      suffixIcon: showIcon
          ? InkWell(
              onTap: () => setState(
                () => passwordVisibility = !passwordVisibility,
              ),
              child: Icon(
                passwordVisibility
                    ? Icons.visibility_outlined
                    : Icons.visibility_off_outlined,
                color: Color(0xFF757575),
                size: 22,
              ),
            )
          : null,
    );
  }
}
