import 'dart:convert';
import 'dart:io';

import 'package:apus_drones_mobile/assets/config.dart';
import 'package:apus_drones_mobile/models/AuthLoginModel.dart';
import 'package:apus_drones_mobile/models/CreateCustomer.dart';
import 'package:apus_drones_mobile/models/CreatePartner.dart';
import 'package:apus_drones_mobile/providers/AuthProvider.dart';
import 'package:http/http.dart' as http;

class AuthService {
  Future<Map<String, dynamic>> signupCustomer(CreateCustomer customer) async {
    print(json.encode(customer.toJson()));
    try {
      http.Response response = await http.post(
          Uri.parse('${Config.apiURL}/authenticate/signup/customer'),
          headers: {"Content-Type": "application/json"},
          body: json.encode(customer.toJson()));

      print('response: ${response.statusCode} - ${response.body}');
      return await AuthProvider().login(response.body);
    } catch (error) {
      print(error);
      return {};
    }
  }

  Future<void> signupPartner(CreatePartner partner) async {
    print(json.encode(partner.toJson()));
    try {
      http.Response response = await http.post(
          Uri.parse('${Config.apiURL}/authenticate/signup/partner'),
          headers: {"Content-Type": "application/json"},
          body: json.encode(partner.toJson()));

      print('response: ${response.statusCode} - ${response.body}');
    } catch (error) {
      print(error);
    }
  }

  Future<Map<String, dynamic>> login(AuthLoginModel loginInfo) async {
    http.Response response = await http.post(
      Uri.parse('${Config.apiURL}/authenticate/login'),
      headers: {"Content-Type": "application/json"},
      body: json.encode(loginInfo.toJson()),
    );

    final status = response.statusCode;
    switch (status) {
      case HttpStatus.ok:
        return this._loginSuccess(response);
      case HttpStatus.forbidden:
        return this._loginForbidden();
      default:
        return {};
    }
  }

  Future<Map<String, dynamic>> _loginSuccess(http.Response response) async {
    return await AuthProvider().login(response.body);
  }

  Map<String, dynamic> _loginForbidden() {
    return {'success': false, 'status': HttpStatus.forbidden};
  }
}
