import 'package:apus_drones_mobile/models/OrderItemModel.dart';
import 'package:apus_drones_mobile/models/CartModel.dart';
import 'package:apus_drones_mobile/models/PartnerModel.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/screens/CartScreen.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:apus_drones_mobile/widgets/WeightAlert.dart';
import 'package:flutter/material.dart';

class AddToCart extends StatelessWidget {
  final int? quantity;
  final OrderService service;
  final ProductModel product;
  AddToCart(
      {Key? key, required this.product, this.quantity, required this.service})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      onPressed: () {
        this._addToCart(context);
      },
      child: Row(
        children: [
          Text(
            "ADICIONAR ",
          ),
          Icon(
            Icons.shopping_cart,
            size: 15,
          ),
        ],
      ),
    );
  }

  _addToCart(BuildContext context) async {
    PartnerModel? partner = product.partner;
    if (partner == null) {
      print('Sem parceiro definido');
      return;
    }

    final _quantity = quantity;
    CartModel order = CartModel(
      partner: partner,
      items: [
        OrderItemModel(
          id: 0,
          quantity: _quantity ?? 1,
          price: product.price,
          product: product,
        )
      ],
    );

    try {
      var cart = await service.myCart();
      if (cart != null && cart.partner.id != partner.id) {
        _showAlert(context,
            "Não é possível adicionar produtos de mais um parceiro ao carrinho. Já há produtos do parceiro ${cart.partner.name}.");
        return;
      }
      await service.addToCart(order);
      Navigator.of(context).pushNamed(CartScreen.routeName);
    } catch (e) {
      WeightAlert.show(context);
    }
  }

  _showAlert(BuildContext context, String message) {
    AlertDialog alertPopUp = AlertDialog(
      title: Text("Erro"),
      content: Text(message),
      actions: [
        ElevatedButton(
            style: ElevatedButton.styleFrom(
                primary: Colors.grey.shade100, onPrimary: Colors.black),
            child: Text("Ok"),
            onPressed: () => Navigator.pop(context, false))
      ],
    );
    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alertPopUp;
      },
    );
  }
}
