import 'package:apus_drones_mobile/constants/Constants.dart';
import 'package:apus_drones_mobile/models/OrderItemModel.dart';
import 'package:apus_drones_mobile/widgets/Price.dart';
import 'package:flutter/material.dart';

class CartItem extends StatefulWidget {
  final OrderItemModel item;
  final String storeName;
  final ValueSetter<OrderItemModel> removeCallback;
  final ValueSetter<OrderItemModel> updateItemCallback;
  final bool Function(OrderItemModel) isWeightValidCallback;

  CartItem({
    Key? key,
    required this.item,
    required this.storeName,
    required this.removeCallback,
    required this.isWeightValidCallback,
    required this.updateItemCallback,
  }) : super(key: key);

  @override
  _CartItemState createState() => _CartItemState();
}

class _CartItemState extends State<CartItem> {
  OrderItemModel? item;
  @override
  void initState() {
    super.initState();
    this.setState(() {
      item = widget.item;
    });
  }

  @override
  Widget build(BuildContext context) {
    var xButton = Align(
      alignment: Alignment.topRight,
      child: Container(
        width: 24,
        height: 24,
        alignment: Alignment.center,
        margin: EdgeInsets.only(right: 10, top: 8),
        child: GestureDetector(
          onTap: () => showAlertDialog(context, item!),
          child: Icon(
            Icons.close,
            color: Colors.white,
          ),
        ),
        decoration: BoxDecoration(
            borderRadius: BorderRadius.all(Radius.circular(4)),
            color: Colors.red),
      ),
    );

    return Stack(
      children: <Widget>[
        Container(
          margin: EdgeInsets.only(left: 16, right: 16, top: 16),
          decoration: BoxDecoration(
              boxShadow: [
                BoxShadow(
                    blurRadius: 5, offset: Offset(2, 3), color: Colors.black26)
              ],
              color: Colors.white,
              borderRadius: BorderRadius.all(Radius.circular(16))),
          child: Row(
            children: <Widget>[
              Container(
                margin: EdgeInsets.only(right: 8, left: 8, top: 8, bottom: 8),
                width: 80,
                height: 80,
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.all(Radius.circular(14)),
                    image: DecorationImage(
                        image: NetworkImage(
                            item!.product.imageUrl ?? IMG_NO_PHOTO))),
              ),
              Expanded(
                child: Container(
                  padding: const EdgeInsets.all(8.0),
                  child: Column(
                    mainAxisSize: MainAxisSize.max,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Container(
                        padding: EdgeInsets.only(right: 8, top: 4),
                        child: Text(
                          item!.product.name,
                          maxLines: 2,
                          softWrap: true,
                          style: TextStyle(
                              fontSize: 16,
                              color: Colors.black,
                              fontWeight: FontWeight.w600),
                        ),
                      ),
                      SizedBox(height: 6),
                      Text(
                        widget.storeName,
                        style: TextStyle(
                            fontSize: 14,
                            color: Colors.grey,
                            fontWeight: FontWeight.w400),
                      ),
                      Container(
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Price(price: item!.price),
                            Padding(
                              padding: const EdgeInsets.all(8.0),
                              child: Container(
                                padding: const EdgeInsets.all(3.0),
                                decoration: BoxDecoration(
                                    color: Colors.grey.shade100,
                                    borderRadius: BorderRadius.circular(8)),
                                child: Row(
                                  mainAxisAlignment: MainAxisAlignment.center,
                                  crossAxisAlignment: CrossAxisAlignment.end,
                                  children: <Widget>[
                                    GestureDetector(
                                      onTap: () {
                                        var newQuantity = item!.quantity - 1;
                                        if (newQuantity <= 0) {
                                          return this
                                              .showAlertDialog(context, item!);
                                        }
                                        var newItem = OrderItemModel(
                                            id: item!.id,
                                            quantity: item!.quantity - 1,
                                            price: item!.price,
                                            product: item!.product);
                                        this.setState(() {
                                          item = newItem;
                                        });
                                        widget.updateItemCallback(item!);
                                      },
                                      child: Icon(
                                        Icons.remove,
                                        size: 24,
                                        color: Colors.grey.shade700,
                                      ),
                                    ),
                                    Container(
                                      padding: const EdgeInsets.only(
                                          bottom: 2, right: 12, left: 12),
                                      child: Text(item != null
                                          ? item!.quantity.toString()
                                          : ""),
                                    ),
                                    GestureDetector(
                                      onTap: () {
                                        var newItem = OrderItemModel(
                                            id: item!.id,
                                            quantity: item!.quantity + 1,
                                            price: item!.price,
                                            product: item!.product);
                                        if (widget
                                            .isWeightValidCallback(item!)) {
                                          this.setState(() {
                                            item = newItem;
                                          });
                                          widget.updateItemCallback(item!);
                                        }
                                      },
                                      child: Icon(
                                        Icons.add,
                                        size: 24,
                                        color: Colors.grey.shade700,
                                      ),
                                    )
                                  ],
                                ),
                              ),
                            )
                          ],
                        ),
                      ),
                    ],
                  ),
                ),
                flex: 100,
              )
            ],
          ),
        ),
        xButton
      ],
    );
  }

  showAlertDialog(BuildContext context, OrderItemModel item) {
    Widget cancelButton = ElevatedButton(
      style: ElevatedButton.styleFrom(
          primary: Colors.grey.shade100, onPrimary: Colors.black),
      child: Text("Cancelar"),
      onPressed: () => Navigator.pop(context, false),
    );
    Widget continueButton = ElevatedButton(
      style: ElevatedButton.styleFrom(primary: Colors.red),
      child: Text("Remover"),
      onPressed: () {
        item.quantity = 0;
        widget.removeCallback(item);
        Navigator.pop(context, false);
      },
    );
    AlertDialog alert = AlertDialog(
      content: Text(
          "Você tem certeza que deseja remover o item '${item.product.name}' ?"),
      actions: [
        cancelButton,
        continueButton,
      ],
    );
    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }
}
