import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:flutter/material.dart';

class FilterOrderStatusMenu extends StatefulWidget {
  final OrderStatus startingStatus;
  final ValueSetter<OrderStatus> changeStatusCallback;
  final List<OrderStatus>? statusesToShow;

  FilterOrderStatusMenu({
    Key? key,
    required this.startingStatus,
    required this.changeStatusCallback,
    this.statusesToShow,
  }) : super(key: key);

  @override
  _FilterOrderStatusMenu createState() {
    return _FilterOrderStatusMenu();
  }
}

class _FilterOrderStatusMenu extends State<FilterOrderStatusMenu> {
  late OrderStatus currentStatus;

  @override
  void initState() {
    super.initState();
    this.setState(() {
      currentStatus = widget.startingStatus;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        height: 30,
        decoration: BoxDecoration(
            border: Border.all(color: currentStatus.color),
            borderRadius: BorderRadius.circular(20),
            color: Colors.grey.shade100),
        padding: const EdgeInsets.symmetric(horizontal: 10),
        child: DropdownButtonHideUnderline(
          child: DropdownButton<OrderStatus>(
            isExpanded: false,
            value: currentStatus,
            items: buildMenuItems(),
            onChanged: (newState) => {
              this.setState(() {
                currentStatus = newState!;
                widget.changeStatusCallback(currentStatus);
              })
            },
          ),
        ));
  }

  List<DropdownMenuItem<OrderStatus>> buildMenuItems() {
    List<OrderStatus> orderStatus = OrderStatus.values;
    final _statusesToShow = widget.statusesToShow;
    return orderStatus
        .where((s) =>
            s == OrderStatus.ALL ||
            _statusesToShow == null ||
            _statusesToShow.length == 0 ||
            _statusesToShow.contains(s))
        .map((s) => DropdownMenuItem<OrderStatus>(
              child: Text(s.label, textAlign: TextAlign.center),
              value: s,
            ))
        .toList();
  }
}
