import 'package:flutter/material.dart';

class QuantityChangerChip extends StatelessWidget {
  final int quantity;
  final int minQuantity;
  final int maxQuantity;
  final VoidCallback onTapMinus;
  final VoidCallback onTapPlus;
  const QuantityChangerChip(
      {Key? key,
      required this.quantity,
      this.minQuantity = 1,
      required this.maxQuantity,
      required this.onTapMinus,
      required this.onTapPlus})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final plusButtonEnabled = quantity < maxQuantity;
    final minusButtonEnabled = quantity > minQuantity;

    return Container(
      width: 150,
      alignment: Alignment.center,
      decoration: BoxDecoration(
        border: Border.all(color: Colors.grey),
        borderRadius: BorderRadius.all(
          Radius.circular(50),
        ),
      ),
      padding: EdgeInsets.symmetric(vertical: 6, horizontal: 20),
      child: Row(
        children: [
          InkWell(
            onTap: minusButtonEnabled ? onTapMinus : null,
            child: Container(
              width: 20,
              alignment: Alignment.center,
              child: Text(
                '-',
                style: TextStyle(
                  fontSize: 20,
                  fontWeight: FontWeight.bold,
                  color:
                      minusButtonEnabled ? Colors.black : Colors.grey.shade400,
                ),
              ),
            ),
          ),
          SizedBox(width: 16),
          Expanded(
            child: Container(
              alignment: Alignment.center,
              child: Text(
                quantity.toString(),
                style: TextStyle(
                  fontSize: 20,
                  // fontWeight: FontWeight.bold,
                ),
              ),
            ),
          ),
          SizedBox(width: 16),
          InkWell(
            onTap: plusButtonEnabled ? onTapPlus : null,
            child: Container(
              width: 20,
              alignment: Alignment.center,
              child: Text(
                '+',
                style: TextStyle(
                  fontSize: 20,
                  fontWeight: FontWeight.bold,
                  color:
                      plusButtonEnabled ? Colors.black : Colors.grey.shade400,
                ),
              ),
            ),
          )
        ],
      ),
    );
  }
}
